#include <locale.h>
#include <config.h>
#include <gnome.h>

#include <gst/player/gstmediaplay.h>
#include <gst/player/gtk-playlist.h>
#include <libgnomevfs/gnome-vfs.h>
#include <gdk/gdkx.h>

GtkWidget *app = NULL;
GstMediaPlay *mplay = NULL;
gboolean app_state_fullscreen = FALSE;

static void
toggle_play_pause (GtkWidget *widget)
{
	GstElementState playstate;

	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	
	if (!gtk_playlist_get_current_mrl (GTK_PLAYLIST(mplay->playlist)))
		return;

	playstate = gst_media_play_get_state (mplay);

	if (playstate == GST_STATE_PLAYING) {
		gst_media_play_set_state (mplay, GST_STATE_PAUSED);
	}
	else {
		gst_media_play_set_state (mplay, GST_STATE_PLAYING);
	}
}

static void
show_about (GtkWidget *widget)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_show_about(mplay);
}

static void
volume_increase (GtkWidget *widget)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_set_volume (mplay, gst_media_play_get_volume (mplay) + 0.1);
}

static void
volume_decrease (GtkWidget *widget)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_set_volume (mplay, gst_media_play_get_volume (mplay) - 0.1);
}

static void
next_media (GtkWidget *widget)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_next ( NULL, mplay);
}
static void
previous_media (GtkWidget *widget)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	gst_media_play_previous ( NULL, mplay);
}

static void
toggle_vis (GtkWidget *widget)
{
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	
	gst_media_play_toggle_visualisation (mplay);
}

/* Fullscreen management stuff */

void
wmspec_change_state (gboolean   add,
                     GdkWindow *window,
                     GdkAtom    state1,
                     GdkAtom    state2)
{
  XEvent xev;

#define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
#define _NET_WM_STATE_ADD           1    /* add/set property */
#define _NET_WM_STATE_TOGGLE        2    /* toggle property  */  
  
  xev.xclient.type = ClientMessage;
  xev.xclient.serial = 0;
  xev.xclient.send_event = True;
  xev.xclient.display = gdk_display;
  xev.xclient.window = GDK_WINDOW_XID (window);
  xev.xclient.message_type = gdk_x11_get_xatom_by_name ("_NET_WM_STATE");
  xev.xclient.format = 32;
  xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
  xev.xclient.data.l[1] = gdk_x11_atom_to_xatom (state1);
  xev.xclient.data.l[2] = gdk_x11_atom_to_xatom (state2);
  
  XSendEvent (gdk_display, GDK_WINDOW_XID (gdk_get_default_root_window ()),
              False,
	      SubstructureRedirectMask | SubstructureNotifyMask,
	      &xev);
}

gboolean
cb_configure (GtkWidget * widget, GdkEventConfigure *event)
{

	/* Window is now effectively fullscreen */
	
	if ( app_state_fullscreen ) {
		gst_media_play_set_display_mode (	mplay,
											GST_MEDIA_PLAY_FULLSCREEN,
											event->width, event->height);
		gtk_widget_hide (GNOME_APP(app)->menubar);
	}
	
	return FALSE;
}

gboolean
cb_fullscreen (GtkWidget * widget, GdkEvent *event)
{

	/* Window is now effectively fullscreen */
	
	if ( (event->window_state.new_window_state == GDK_WINDOW_STATE_FULLSCREEN)
		|| (event->window_state.new_window_state == 20) )	{
		app_state_fullscreen = TRUE;	
	}
	else {
		app_state_fullscreen = FALSE;
		gst_media_play_set_display_mode (	mplay,
											GST_MEDIA_PLAY_NORMAL,
											0, 0);
		gtk_widget_show (GNOME_APP(app)->menubar);
	}
	
	return TRUE;
}

static void
toggle_fullscreen (GtkWidget *widget)
{
	GstMediaPlayMode display_mode;
	g_return_if_fail(GST_IS_MEDIA_PLAY(mplay));
	
	display_mode = gst_media_play_get_display_mode (mplay);
	if (display_mode == GST_MEDIA_PLAY_FULLSCREEN){
		wmspec_change_state (FALSE,
			GTK_WIDGET (app)->window,
			gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", FALSE),
			GDK_NONE);
	}
	else {
		wmspec_change_state (TRUE,
			GTK_WIDGET (app)->window,
			gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN", FALSE),
			GDK_NONE);
	}
}

/* Key press events management */

gint
handle_keypress (	GtkWidget *widget,
					GdkEventKey *event)
{
	g_return_val_if_fail (widget != NULL, 0);
	g_return_val_if_fail (GST_IS_MEDIA_PLAY (mplay), 0);
	
	if ( (event->keyval == GDK_Escape) || (event->keyval == GDK_f) ) {
		toggle_fullscreen (widget);
		return TRUE;
	}
	else
		return gst_media_play_handle_keypress (widget, event, mplay);
}

static void
cb_open (GtkWidget *widget)
{
	GtkWidget *fs = NULL;
	static gchar *path = NULL;
	gint response;
	
	g_return_if_fail (GST_IS_MEDIA_PLAY(mplay));

	fs = gtk_file_selection_new (_("Select files"));
	gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (fs), TRUE);
	if (path != NULL)
	{
		gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs),
				path);
		g_free (path);
		path = NULL;
	}
	
	response = gtk_dialog_run (GTK_DIALOG (fs));
	gtk_widget_hide (fs);

	if (response == GTK_RESPONSE_OK) {
		gchar **filenames;

		filenames = gtk_file_selection_get_selections (GTK_FILE_SELECTION (fs));
		
		if (filenames[0]) {
			gchar *tmp;
			gint i;
			
			gst_media_play_ignore_playlist_changes (mplay, TRUE);
			
			gtk_playlist_clear (GTK_PLAYLIST(mplay->playlist));
			
			for (i=0; filenames[i] != NULL; i++)
				gtk_playlist_add_mrl (	GTK_PLAYLIST(mplay->playlist),
										filenames[i], NULL);

			tmp = g_path_get_dirname (filenames[0]);
			path = g_strconcat (tmp, G_DIR_SEPARATOR_S, NULL);
			g_free (tmp);
		}
		g_strfreev (filenames);

		gst_media_play_ignore_playlist_changes (mplay, FALSE);
		
		gst_media_play_set_location (
				mplay,
				gtk_playlist_get_current_mrl (GTK_PLAYLIST(mplay->playlist)));
		
		toggle_play_pause (GTK_WIDGET(mplay));
		
	}

	gtk_widget_destroy (fs);
}

static void
cb_open_location (GtkWidget *widget)
{
	GtkWidget *dialog, *textbox;
	gint response;
	static gchar *last_location = NULL;

	dialog = gnome_dialog_new (
					_("Open Location"),
					GNOME_STOCK_BUTTON_CANCEL,
					_("Enqueue"),
					GNOME_STOCK_BUTTON_OK,
					NULL);
	
	gnome_dialog_set_close(GNOME_DIALOG(dialog), TRUE);
  	gnome_dialog_close_hides(GNOME_DIALOG(dialog), TRUE);
	gnome_dialog_set_default(GNOME_DIALOG(dialog), 2);
	gnome_dialog_set_parent(	GNOME_DIALOG(dialog),
								GTK_WINDOW(gtk_widget_get_toplevel (app)));
	
	textbox = gtk_entry_new();
	if (last_location) {
		g_message ("last_location is %s", last_location);
		gtk_entry_set_text(GTK_ENTRY(textbox), last_location);
		gtk_editable_select_region(GTK_EDITABLE(textbox), 0, -1);
	}
	gtk_entry_set_width_chars(GTK_ENTRY(textbox), 80);
	gtk_box_pack_start(	GTK_BOX(GNOME_DIALOG(dialog)->vbox),
						textbox,
						TRUE, TRUE, GNOME_PAD);
	gnome_dialog_editable_enters(GNOME_DIALOG(dialog), GTK_EDITABLE(textbox));
	
	gtk_widget_show_all(dialog);
	
	response = gnome_dialog_run(GNOME_DIALOG(dialog));
	
	switch (response) {
		case 2:
			gtk_playlist_clear (GTK_PLAYLIST(mplay->playlist));
		case 1:
			gtk_playlist_add_mrl (	GTK_PLAYLIST(mplay->playlist),
							gtk_entry_get_text(GTK_ENTRY(textbox)), NULL);
			gst_media_play_set_location (
				mplay,
				gtk_playlist_get_current_mrl (GTK_PLAYLIST(mplay->playlist)));
			if (last_location) g_free (last_location);
			last_location = g_strdup(gtk_entry_get_text(GTK_ENTRY(textbox)));
			break;
		default:
			break;
	}
	
	gtk_widget_destroy(dialog); 
}

gboolean
cb_destroy (GtkWidget * widget, GdkEvent *event)
{
	gtk_widget_destroy (GTK_WIDGET(mplay));
	gtk_main_quit ();
	
	return TRUE;
}

static void
current_location (GstMediaPlay *mplay, gchar *location, gchar *location_short, GtkWidget *window)
{
	gtk_window_set_title (GTK_WINDOW (window),location_short);
}



static GnomeUIInfo file_menu [] = 
{
	GNOMEUIINFO_MENU_OPEN_ITEM (cb_open, NULL),
	{
		GNOME_APP_UI_ITEM, N_("Open _Location"),
		NULL,
		cb_open_location, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_L, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_EXIT_ITEM (cb_destroy, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo control_menu [] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Play / Pause"),
		NULL,
		toggle_play_pause, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_p, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Increase Volume"),
		NULL,
		volume_increase, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_Up, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Decrease Volume"),
		NULL,
		volume_decrease, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_Down, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Next Media"),
		NULL,
		next_media, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_Right, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("Previous Media"),
		NULL,
		previous_media, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_Left, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("Toggle visualisation"),
		NULL,
		toggle_vis, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_v, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo view_menu [] =
{
/*	{
		GNOME_APP_UI_ITEM, N_("_Half Size"),
		NULL,
		half_size, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_1, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Original Size"),
		NULL,
		full_size, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_2, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Double Size"),
		NULL,
		double_size, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_3, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},*/
	{
		GNOME_APP_UI_ITEM, N_("_Full Screen"),
		NULL,
		toggle_fullscreen, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		GDK_f, (GdkModifierType) GDK_CONTROL_MASK, NULL
	},
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu [] =
{
	GNOMEUIINFO_MENU_ABOUT_ITEM (show_about, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo main_menu [] = 
{
	GNOMEUIINFO_MENU_FILE_TREE (file_menu),
        {
		GNOME_APP_UI_SUBTREE, N_("_Control"), NULL, control_menu, NULL, NULL, 
		(GnomeUIPixmapType) 0, NULL, 0, (GdkModifierType) 0, NULL 
	},
	GNOMEUIINFO_MENU_VIEW_TREE (view_menu),
	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
	GNOMEUIINFO_END
};

int
main (int argc, char **argv)
{
	gboolean play_on_startup = FALSE;
	
	struct poptOption options[] = {
	  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, 
	    NULL, 0, "GStreamer", NULL },
	    POPT_TABLEEND
	};
	GnomeProgram *p;
	poptContext context;
	const gchar **argvn;
	char *file;

	options[0].arg = (void *) gst_init_get_popt_table ();
	
	/* Using opt as default scheduler */
	gst_scheduler_factory_set_default_name ("opt");
	
	if (! (p = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
		                       argc, argv,
		   		       GNOME_PARAM_POPT_TABLE, options,
				       GNOME_PARAM_HUMAN_READABLE_NAME, _("Gst-Player"),
                                       GNOME_PARAM_APP_DATADIR, DATADIR,
				       NULL)))
	  g_error ("gnome_program_init failed");

	/* i18n initialization */
	bindtextdomain (PACKAGE, LOCALEDIR);
	bind_textdomain_codeset (PACKAGE, "UTF-8");
	textdomain (PACKAGE);

	if (!gnome_vfs_init ()) {
		g_print ("Could not initialize GnomeVFS\n");
		return 1;
	}
	
	/* gnome window */
	app = gnome_app_new ("gst-player", "GStreamer Player");
	g_return_val_if_fail (app != NULL, -1);

	gtk_window_set_default_size (GTK_WINDOW (app), 300, -1);
	gtk_window_set_resizable (GTK_WINDOW (app), TRUE);
	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, 
	                                  "gst-player-48.png", TRUE, NULL);
	if (file != NULL)
	{
		gnome_window_icon_set_default_from_file (file);
		g_free (file);
	}

	mplay = gst_media_play_new ();
	/* if it didn't get created properly, fail */
	if (!GST_IS_MEDIA_PLAY (mplay))
		return -1;

	gnome_app_create_menus_with_data (GNOME_APP (app), main_menu, mplay);
	gnome_app_set_contents (GNOME_APP (app), GTK_WIDGET(mplay));
	
	g_signal_connect (G_OBJECT (app), "window-state-event",
			G_CALLBACK (cb_fullscreen), mplay);
	g_signal_connect (G_OBJECT (app), "configure-event",
			G_CALLBACK (cb_configure), mplay);
	g_signal_connect (G_OBJECT (app), "delete-event",
			G_CALLBACK (cb_destroy), mplay);
	g_signal_connect (G_OBJECT (app), "key-press-event",
			G_CALLBACK (handle_keypress), mplay);
	g_signal_connect (G_OBJECT (mplay), "display_mode_change",
			G_CALLBACK (toggle_fullscreen), app);
	g_signal_connect (G_OBJECT (mplay), "current_location",
			G_CALLBACK (current_location), app);

	gtk_widget_show (GTK_WIDGET(mplay));
	gtk_widget_show (app);

	g_object_get (p, "popt-context", &context, NULL);
	argvn = poptGetArgs (context);
	if (argvn) {
		gint i=0;
		while (argvn[i] != NULL) {
			gtk_playlist_add_mrl (	GTK_PLAYLIST(mplay->playlist),
										argvn[i], NULL);
			i++;
		}
		gst_media_play_set_location (
				mplay,
				gtk_playlist_get_current_mrl (GTK_PLAYLIST(mplay->playlist)));
		
		/* check if we should play on startup */
		if (!gst_media_gconf_get_boolean ("play-on-startup", &play_on_startup))
			play_on_startup = FALSE;
		if (play_on_startup) {
			toggle_play_pause (GTK_WIDGET(mplay));
		}
		
	}

	gtk_main ();

	return 0;
}
